@@ -524,11 +524,11 @@ static int new_session_cb(SSL *ssl __attribute__((unused)),
 
         session_id = SSL_SESSION_get_id(sess, &session_id_length);
         if (session_id_length) {
-	    do {
-	        ret = cyrusdb_store(sessdb, (const char *) session_id,
-			        session_id_length,
-			        (const char *) data, len + sizeof(time_t), NULL);
-	    } while (ret == CYRUSDB_AGAIN);
+            do {
+                ret = cyrusdb_store(sessdb, (const char *) session_id,
+                                session_id_length,
+                                (const char *) data, len + sizeof(time_t), NULL);
+            } while (ret == CYRUSDB_AGAIN);
         }
     }
 
@@ -1386,13 +1386,7 @@ EXPORTED int tls_prune_sessions(void)
     }
 
     ret = cyrusdb_open(DB, fname, 0, &sessdb);
-    if (ret != CYRUSDB_OK) {
-        syslog(LOG_ERR, "DBERROR: opening %s: %s",
-               fname, cyrusdb_strerror(ret));
-        free(tofree);
-        return 1;
-    }
-    else {
+    if (ret == CYRUSDB_OK) {
         /* check each session in our database */
         sess_dbopen = 1;
         prock.count = prock.deletions = 0;
@@ -1404,10 +1398,19 @@ EXPORTED int tls_prune_sessions(void)
         syslog(LOG_NOTICE, "tls_prune: purged %d out of %d entries",
                prock.deletions, prock.count);
     }
+    else if (ret == CYRUSDB_NOTFOUND) {
+        syslog(LOG_NOTICE, "tls_prune: %s not found, nothing to do", fname);
+        ret = 0;
+    }
+    else {
+        syslog(LOG_ERR, "DBERROR: opening %s: %s",
+               fname, cyrusdb_strerror(ret));
+    }
 
-    free(tofree);
+    if (tofree)
+	free(tofree);
 
-    return 0;
+    return ret;
 }
 
 /* fill string buffer with info about tls connection */
