@@ -54,6 +54,7 @@
 #include "util.h"
 #include "version.h"
 #include "xcal.h"
+#include "xml_support.h"
 
 
 /*
@@ -66,28 +67,28 @@ const char *icalproperty_value_kind_as_string(icalproperty *prop)
 
     val_param = icalproperty_get_first_parameter(prop, ICAL_VALUE_PARAMETER);
     if (val_param) {
-	/* Use the kind specified in the VALUE param */
-	kind = icalparameter_value_to_value_kind(
-	    icalparameter_get_value(val_param));
+        /* Use the kind specified in the VALUE param */
+        kind = icalparameter_value_to_value_kind(
+            icalparameter_get_value(val_param));
     }
 
     if (kind == ICAL_NO_VALUE) {
-	icalvalue *value = icalproperty_get_value(prop);
+        icalvalue *value = icalproperty_get_value(prop);
 
-	if (value) {
-	    /* Use the kind determined from the property value */
-	    kind = icalvalue_isa(value);
-	}
+        if (value) {
+            /* Use the kind determined from the property value */
+            kind = icalvalue_isa(value);
+        }
     }
 
     if (kind == ICAL_NO_VALUE) {
-	/* Use the default kind for the property */
-	kind = icalproperty_kind_to_value_kind(icalproperty_isa(prop));
+        /* Use the default kind for the property */
+        kind = icalproperty_kind_to_value_kind(icalproperty_isa(prop));
     }
 
     switch (kind) {
     case ICAL_X_VALUE:
-	return "unknown";
+        return "unknown";
 
     case ICAL_ACTION_VALUE:
     case ICAL_CARLEVEL_VALUE:
@@ -97,12 +98,12 @@ const char *icalproperty_value_kind_as_string(icalproperty *prop)
     case ICAL_QUERYLEVEL_VALUE:
     case ICAL_STATUS_VALUE:
     case ICAL_TRANSP_VALUE:
-	return "text";
+        return "text";
 
     default:
-	return icalvalue_kind_to_string(kind);
+        return icalvalue_kind_to_string(kind);
     }
-} 
+}
 
 
 /*
@@ -118,7 +119,7 @@ const char *icaltime_as_iso_string(const struct icaltimetype tt)
     else fmt = "%04d-%02d-%02dT%02d:%02d:%02d";
 
     snprintf(str, sizeof(str), fmt, tt.year, tt.month, tt.day,
-	     tt.hour, tt.minute, tt.second);
+             tt.hour, tt.minute, tt.second);
 
     return str;
 }
@@ -128,7 +129,7 @@ const char *icaltime_as_iso_string(const struct icaltimetype tt)
  * Construct an ISO.8601.2004 string for an iCalendar UTC Offset.
  */
 const char *icalvalue_utcoffset_as_iso_string(const icalvalue* value)
-{    
+{
     static char str[10];
     const char *fmt;
     int off, h, m, s;
@@ -156,9 +157,9 @@ const char *icalvalue_utcoffset_as_iso_string(const icalvalue* value)
  * Add iCalendar recur-rule-parts to a structured element.
  */
 void icalrecurrencetype_add_as_xxx(struct icalrecurrencetype *recur, void *obj,
-				   void (*add_int)(void *, const char *, int),
-				   void (*add_str)(void *, const char *,
-						   const char *))
+                                   void (*add_int)(void *, const char *, int),
+                                   void (*add_str)(void *, const char *,
+                                                   const char *))
 {
     char *rrule, *rpart;
     tok_t rparts;
@@ -208,7 +209,7 @@ void icalrecurrencetype_add_as_xxx(struct icalrecurrencetype *recur, void *obj,
  * Add an XML element for an iCalendar Period.
  */
 static void icalperiodtype_add_as_xml_element(xmlNodePtr xtype,
-					      struct icalperiodtype p)
+                                              struct icalperiodtype p)
 {
     const char *start;
     const char *end;
@@ -217,12 +218,12 @@ static void icalperiodtype_add_as_xml_element(xmlNodePtr xtype,
     xmlNewTextChild(xtype, NULL, BAD_CAST "start", BAD_CAST start);
 
     if (!icaltime_is_null_time(p.end)) {
-	end = icaltime_as_iso_string(p.end);
-	xmlNewTextChild(xtype, NULL, BAD_CAST "end", BAD_CAST end);
+        end = icaltime_as_iso_string(p.end);
+        xmlNewTextChild(xtype, NULL, BAD_CAST "end", BAD_CAST end);
     }
     else {
-	end = icaldurationtype_as_ical_string(p.duration);
-	xmlNewTextChild(xtype, NULL, BAD_CAST "duration", BAD_CAST end);
+        end = icaldurationtype_as_ical_string(p.duration);
+        xmlNewTextChild(xtype, NULL, BAD_CAST "duration", BAD_CAST end);
     }
 }
 
@@ -231,7 +232,7 @@ static void icalperiodtype_add_as_xml_element(xmlNodePtr xtype,
  * Add an iCalendar recur-rule-part to a XML recur element.
  */
 static void icalrecur_add_string_as_xml_element(void *xrecur, const char *rpart,
-						const char *s)
+                                                const char *s)
 {
     xmlNewTextChild((xmlNodePtr) xrecur, NULL, BAD_CAST rpart, BAD_CAST s);
 }
@@ -252,45 +253,47 @@ static xmlNodePtr icalparameter_as_xml_element(icalparameter *param)
     switch (kind) {
     case ICAL_X_PARAMETER:
         kind_string = icalparameter_get_xname(param);
-	break;
+        break;
 
 #ifdef HAVE_IANA_PARAMS
     case ICAL_IANA_PARAMETER:
-	kind_string = icalparameter_get_iana_name(param);
-	break;
+        kind_string = icalparameter_get_iana_name(param);
+        break;
 #endif
 
     default:
-	kind_string = icalparameter_kind_to_string(kind);
-	if (kind_string) break;
+        kind_string = icalparameter_kind_to_string(kind);
+        if (kind_string) break;
+
+        GCC_FALLTHROUGH
 
     case ICAL_NO_PARAMETER:
     case ICAL_ANY_PARAMETER:
-	    icalerror_set_errno(ICAL_BADARG_ERROR);
-	    return NULL;
+            icalerror_set_errno(ICAL_BADARG_ERROR);
+            return NULL;
     }
 
     /* Get value type */
     switch (kind) {
     case ICAL_ALTREP_PARAMETER:
     case ICAL_DIR_PARAMETER:
-	type_string = "uri";
-	break;
+        type_string = "uri";
+        break;
 
     case ICAL_DELEGATEDFROM_PARAMETER:
     case ICAL_DELEGATEDTO_PARAMETER:
     case ICAL_MEMBER_PARAMETER:
     case ICAL_SENTBY_PARAMETER:
-	type_string = "cal-address";
-	break;
+        type_string = "cal-address";
+        break;
 
     case ICAL_RSVP_PARAMETER:
-	type_string = "boolean";
-	break;
+        type_string = "boolean";
+        break;
 
     default:
-	type_string = "text";
-	break;
+        type_string = "text";
+        break;
     }
 
     /* XXX  Need to handle multi-valued parameters */
@@ -310,8 +313,8 @@ static xmlNodePtr icalparameter_as_xml_element(icalparameter *param)
  * Add the proper XML element for an iCalendar value.
  */
 static void icalproperty_add_value_as_xml_element(xmlNodePtr xprop,
-						  icalproperty *prop)
-						  
+                                                  icalproperty *prop)
+
 {
     const char *type, *str = NULL;
     xmlNodePtr xtype;
@@ -320,7 +323,7 @@ static void icalproperty_add_value_as_xml_element(xmlNodePtr xprop,
 
     /* Add type */
     type = lcase(icalmemory_tmp_copy(
-		     icalproperty_value_kind_as_string(prop)));
+                     icalproperty_value_kind_as_string(prop)));
     xtype = xmlNewChild(xprop, NULL, BAD_CAST type, NULL);
 
 
@@ -329,43 +332,43 @@ static void icalproperty_add_value_as_xml_element(xmlNodePtr xprop,
 
     switch (icalvalue_isa(value)) {
     case ICAL_DATE_VALUE:
-	str = icaltime_as_iso_string(icalvalue_get_date(value));
-	break;
+        str = icaltime_as_iso_string(icalvalue_get_date(value));
+        break;
 
     case ICAL_DATETIME_VALUE:
-	str = icaltime_as_iso_string(icalvalue_get_datetime(value));
-	break;
+        str = icaltime_as_iso_string(icalvalue_get_datetime(value));
+        break;
 
     case ICAL_DATETIMEPERIOD_VALUE: {
-	struct icaldatetimeperiodtype dtp =
-	    icalvalue_get_datetimeperiod(value);
+        struct icaldatetimeperiodtype dtp =
+            icalvalue_get_datetimeperiod(value);
 
-	if (!icaltime_is_null_time(dtp.time)) {
-	    str = icaltime_as_iso_string(dtp.time);
-	    break;
-	}
-	else {
-	    icalperiodtype_add_as_xml_element(xtype, dtp.period);
-	    return;
-	}
+        if (!icaltime_is_null_time(dtp.time)) {
+            str = icaltime_as_iso_string(dtp.time);
+            break;
+        }
+        else {
+            icalperiodtype_add_as_xml_element(xtype, dtp.period);
+            return;
+        }
     }
 
     case ICAL_GEO_VALUE: {
-	struct icalgeotype geo = icalvalue_get_geo(value);
+        struct icalgeotype geo = icalvalue_get_geo(value);
 
-	snprintf(buf, sizeof(buf), "%f", geo.lat);
-	xmlNewTextChild(xtype, NULL, BAD_CAST "latitude", BAD_CAST buf);
-	snprintf(buf, sizeof(buf), "%f", geo.lon);
-	xmlNewTextChild(xtype, NULL, BAD_CAST "longitude", BAD_CAST buf);
-	return;
+        snprintf(buf, sizeof(buf), "%f", geo.lat);
+        xmlNewTextChild(xtype, NULL, BAD_CAST "latitude", BAD_CAST buf);
+        snprintf(buf, sizeof(buf), "%f", geo.lon);
+        xmlNewTextChild(xtype, NULL, BAD_CAST "longitude", BAD_CAST buf);
+        return;
     }
 
     case ICAL_PERIOD_VALUE:
-	icalperiodtype_add_as_xml_element(xtype, icalvalue_get_period(value));
-	return;
+        icalperiodtype_add_as_xml_element(xtype, icalvalue_get_period(value));
+        return;
 
     case ICAL_RECUR_VALUE: {
-	struct icalrecurrencetype recur = icalvalue_get_recur(value);
+        struct icalrecurrencetype recur = icalvalue_get_recur(value);
 
         icalrecurrencetype_add_as_xxx(&recur, xtype, NULL,
                                       &icalrecur_add_string_as_xml_element);
@@ -373,64 +376,64 @@ static void icalproperty_add_value_as_xml_element(xmlNodePtr xprop,
     }
 
     case ICAL_REQUESTSTATUS_VALUE: {
-	struct icalreqstattype stat = icalvalue_get_requeststatus(value);
-	
-	if (!stat.desc) stat.desc = icalenum_reqstat_desc(stat.code);
+        struct icalreqstattype stat = icalvalue_get_requeststatus(value);
 
-	snprintf(buf, sizeof(buf), "%u.%u",
-		 icalenum_reqstat_major(stat.code),
-		 icalenum_reqstat_minor(stat.code));
-	xmlNewTextChild(xtype, NULL, BAD_CAST "code", BAD_CAST buf);
-	xmlNewTextChild(xtype, NULL, BAD_CAST "description", BAD_CAST stat.desc);
-	if (stat.debug)
-	    xmlNewTextChild(xtype, NULL, BAD_CAST "data", BAD_CAST stat.debug);
+        if (!stat.desc) stat.desc = icalenum_reqstat_desc(stat.code);
 
-	return;
+        snprintf(buf, sizeof(buf), "%u.%u",
+                 icalenum_reqstat_major(stat.code),
+                 icalenum_reqstat_minor(stat.code));
+        xmlNewTextChild(xtype, NULL, BAD_CAST "code", BAD_CAST buf);
+        xmlNewTextChild(xtype, NULL, BAD_CAST "description", BAD_CAST stat.desc);
+        if (stat.debug)
+            xmlNewTextChild(xtype, NULL, BAD_CAST "data", BAD_CAST stat.debug);
+
+        return;
     }
 
     case ICAL_TRIGGER_VALUE: {
-	struct icaltriggertype trig = icalvalue_get_trigger(value);
+        struct icaltriggertype trig = icalvalue_get_trigger(value);
 
-	if (!icaltime_is_null_time(trig.time))
-	    str = icaltime_as_iso_string(trig.time);
-	else
-	    str = icaldurationtype_as_ical_string(trig.duration);
-	break;
+        if (!icaltime_is_null_time(trig.time))
+            str = icaltime_as_iso_string(trig.time);
+        else
+            str = icaldurationtype_as_ical_string(trig.duration);
+        break;
     }
 
     case ICAL_UTCOFFSET_VALUE:
-	str = icalvalue_utcoffset_as_iso_string(value);
-	break;
+        str = icalvalue_utcoffset_as_iso_string(value);
+        break;
 
     default:
-	str = icalvalue_as_ical_string(value);
+        str = icalvalue_as_ical_string(value);
 
-	switch (icalproperty_isa(prop)) {
-	case ICAL_CATEGORIES_PROPERTY:
-	case ICAL_RESOURCES_PROPERTY:
-	case ICAL_POLLPROPERTIES_PROPERTY:
-	    if (strchr(str, ',')) {
-		/* Handle multi-valued properties */
-		tok_t tok;
+        switch (icalproperty_isa(prop)) {
+        case ICAL_CATEGORIES_PROPERTY:
+        case ICAL_RESOURCES_PROPERTY:
+        case ICAL_POLLPROPERTIES_PROPERTY:
+            if (strchr(str, ',')) {
+                /* Handle multi-valued properties */
+                tok_t tok;
 
-		tok_init(&tok, str, ",", TOK_TRIMLEFT|TOK_TRIMRIGHT|TOK_EMPTY);
-		str = tok_next(&tok);
-		xmlAddChild(xtype, xmlNewText(BAD_CAST str));
+                tok_init(&tok, str, ",", TOK_TRIMLEFT|TOK_TRIMRIGHT|TOK_EMPTY);
+                str = tok_next(&tok);
+                xmlAddChild(xtype, xmlNewText(BAD_CAST str));
 
-		while ((str = tok_next(&tok))) {
-		    if (*str) {
-			xtype = xmlNewChild(xprop, NULL, BAD_CAST type, NULL);
-			xmlAddChild(xtype, xmlNewText(BAD_CAST str));
-		    }
-		}
-		tok_fini(&tok);
-		return;
-	    }
+                while ((str = tok_next(&tok))) {
+                    if (*str) {
+                        xtype = xmlNewChild(xprop, NULL, BAD_CAST type, NULL);
+                        xmlAddChild(xtype, xmlNewText(BAD_CAST str));
+                    }
+                }
+                tok_fini(&tok);
+                return;
+            }
 
-	default: break;
-	}
+        default: break;
+        }
 
-	break;
+        break;
     }
 
     if (str) xmlAddChild(xtype, xmlNewText(BAD_CAST str));
@@ -441,43 +444,43 @@ static void icalproperty_add_value_as_xml_element(xmlNodePtr xprop,
  * Construct an XML element for an iCalendar property.
  */
 static xmlNodePtr icalproperty_as_xml_element(icalproperty *prop)
-{   
+{
     icalproperty_kind prop_kind;
-    const char *x_name, *property_name = NULL; 
+    const char *x_name, *property_name = NULL;
     icalparameter *param;
     xmlNodePtr xprop, xparams = NULL;
 
     if (!prop) return NULL;
-    
+
     prop_kind = icalproperty_isa(prop);
     x_name = icalproperty_get_x_name(prop);
 
     if (prop_kind == ICAL_X_PROPERTY && x_name)
-	property_name = x_name;
+        property_name = x_name;
     else
-	property_name = icalproperty_kind_to_string(prop_kind);
+        property_name = icalproperty_kind_to_string(prop_kind);
 
     if (!property_name) {
-	icalerror_warn("Got a property of an unknown kind.");
-	return NULL;
+        icalerror_warn("Got a property of an unknown kind.");
+        return NULL;
     }
 
     /* Create property */
     xprop = xmlNewNode(NULL,
-		       BAD_CAST lcase(icalmemory_tmp_copy(property_name)));
+                       BAD_CAST lcase(icalmemory_tmp_copy(property_name)));
 
 
     /* Add parameters */
     for (param = icalproperty_get_first_parameter(prop, ICAL_ANY_PARAMETER);
-	 param != 0;
-	 param = icalproperty_get_next_parameter(prop, ICAL_ANY_PARAMETER)) {
+         param != 0;
+         param = icalproperty_get_next_parameter(prop, ICAL_ANY_PARAMETER)) {
 
-	if (icalparameter_isa(param) == ICAL_VALUE_PARAMETER) continue;
+        if (icalparameter_isa(param) == ICAL_VALUE_PARAMETER) continue;
 
-	if (!xparams)
-	    xparams = xmlNewChild(xprop, NULL, BAD_CAST "parameters", NULL);
+        if (!xparams)
+            xparams = xmlNewChild(xprop, NULL, BAD_CAST "parameters", NULL);
 
-	xmlAddChild(xparams, icalparameter_as_xml_element(param));
+        xmlAddChild(xparams, icalparameter_as_xml_element(param));
     }
 
 
@@ -504,44 +507,44 @@ static xmlNodePtr icalcomponent_as_xml_element(icalcomponent *comp)
     kind = icalcomponent_isa(comp);
     switch (kind) {
     case ICAL_NO_COMPONENT:
-	return NULL;
-	break;
+        return NULL;
+        break;
 
     case ICAL_X_COMPONENT:
-	kind_string = ""; //comp->x_name;
-	break;
+        kind_string = ""; //comp->x_name;
+        break;
 
     default:
-	kind_string = icalcomponent_kind_to_string(kind);
+        kind_string = icalcomponent_kind_to_string(kind);
     }
 
 
     /* Create component */
     xcomp = xmlNewNode(NULL,
-		       BAD_CAST lcase(icalmemory_tmp_copy(kind_string)));
+                       BAD_CAST lcase(icalmemory_tmp_copy(kind_string)));
 
 
     /* Add properties */
     for (p = icalcomponent_get_first_property(comp, ICAL_ANY_PROPERTY);
-	 p;
-	 p = icalcomponent_get_next_property(comp, ICAL_ANY_PROPERTY)) {
+         p;
+         p = icalcomponent_get_next_property(comp, ICAL_ANY_PROPERTY)) {
 
-	if (!xprops)
-	    xprops = xmlNewChild(xcomp, NULL, BAD_CAST "properties", NULL);
+        if (!xprops)
+            xprops = xmlNewChild(xcomp, NULL, BAD_CAST "properties", NULL);
 
-	xmlAddChild(xprops, icalproperty_as_xml_element(p));
+        xmlAddChild(xprops, icalproperty_as_xml_element(p));
     }
 
 
     /* Add sub-components */
     for (c = icalcomponent_get_first_component(comp, ICAL_ANY_COMPONENT);
-	 c;
-	 c = icalcomponent_get_next_component(comp, ICAL_ANY_COMPONENT)) {
+         c;
+         c = icalcomponent_get_next_component(comp, ICAL_ANY_COMPONENT)) {
 
-	if (!xsubs)
-	    xsubs = xmlNewChild(xcomp, NULL, BAD_CAST "components", NULL);
+        if (!xsubs)
+            xsubs = xmlNewChild(xcomp, NULL, BAD_CAST "components", NULL);
 
-	xmlAddChild(xsubs, icalcomponent_as_xml_element(c));
+        xmlAddChild(xsubs, icalcomponent_as_xml_element(c));
     }
 
     return xcomp;
@@ -551,8 +554,9 @@ static xmlNodePtr icalcomponent_as_xml_element(icalcomponent *comp)
 /*
  * Construct a xcal string for an iCalendar component.
  */
-char *icalcomponent_as_xcal_string(icalcomponent *ical)
+struct buf *icalcomponent_as_xcal_string(icalcomponent *ical)
 {
+    struct buf *ret;
     xmlDocPtr doc;
     xmlNodePtr root, xcomp;
     xmlChar *buf;
@@ -570,23 +574,26 @@ char *icalcomponent_as_xcal_string(icalcomponent *ical)
     xmlAddChild(root, xcomp);
 
     if (!xmlStrcmp(xcomp->name, BAD_CAST "vcalendar")) {
-	/* Complete iCalendar stream */
-	xmlDocDumpFormatMemoryEnc(doc, &buf, &bufsiz, "utf-8",
-				  config_httpprettytelemetry);
+        /* Complete iCalendar stream */
+        xmlDocDumpFormatMemoryEnc(doc, &buf, &bufsiz, "utf-8",
+                                  config_httpprettytelemetry);
     }
     else {
-	/* Single iCalendar object */
-	xmlBufferPtr xbuf = xmlBufferCreate();
+        /* Single iCalendar object */
+        xmlBufferPtr xbuf = xmlBufferCreate();
 
-	bufsiz = xmlNodeDump(xbuf, doc, xcomp,
-			     0, config_httpprettytelemetry);
-	buf = xmlBufferDetach(xbuf);
-	xmlBufferFree(xbuf);
+        bufsiz = xmlNodeDump(xbuf, doc, xcomp,
+                             0, config_httpprettytelemetry);
+        buf = xmlBufferDetach(xbuf);
+        xmlBufferFree(xbuf);
     }
 
     xmlFreeDoc(doc);
 
-    return (char *) buf;
+    ret = buf_new();
+    buf_initm(ret, (char *) buf, bufsiz);
+
+    return ret;
 }
 
 
@@ -605,7 +612,7 @@ static void append_byrule(char *byrule, struct buf *vals, struct buf *rrule)
  * Construct an iCalendar property value from XML content.
  */
 static icalvalue *xml_element_to_icalvalue(xmlNodePtr xtype,
-					   icalvalue_kind kind)
+                                           icalvalue_kind kind)
 {
     icalvalue *value = NULL;
     xmlNodePtr node;
@@ -614,90 +621,90 @@ static icalvalue *xml_element_to_icalvalue(xmlNodePtr xtype,
     switch (kind) {
 
     case ICAL_GEO_VALUE: {
-	struct icalgeotype geo;
+        struct icalgeotype geo;
 
-	node = xmlFirstElementChild(xtype);
-	if (!node) {
-	    syslog(LOG_WARNING, "Missing <latitude> XML element");
-	    break;
-	}
-	else if (xmlStrcmp(node->name, BAD_CAST "latitude")) {
-	    syslog(LOG_WARNING,
-		   "Expected <latitude> XML element, received %s", node->name);
-	    break;
-	}
+        node = xmlFirstElementChild(xtype);
+        if (!node) {
+            syslog(LOG_WARNING, "Missing <latitude> XML element");
+            break;
+        }
+        else if (xmlStrcmp(node->name, BAD_CAST "latitude")) {
+            syslog(LOG_WARNING,
+                   "Expected <latitude> XML element, received %s", node->name);
+            break;
+        }
 
-	content = xmlNodeGetContent(node);
-	geo.lat = atof((const char *) content);
+        content = xmlNodeGetContent(node);
+        geo.lat = atof((const char *) content);
 
-	node = xmlNextElementSibling(node);
-	if (!node) {
-	    syslog(LOG_WARNING, "Missing <longitude> XML element");
-	    break;
-	}
-	else if (xmlStrcmp(node->name, BAD_CAST "longitude")) {
-	    syslog(LOG_WARNING,
-		   "Expected <longitude> XML element, received %s", node->name);
-	    break;
-	}
+        node = xmlNextElementSibling(node);
+        if (!node) {
+            syslog(LOG_WARNING, "Missing <longitude> XML element");
+            break;
+        }
+        else if (xmlStrcmp(node->name, BAD_CAST "longitude")) {
+            syslog(LOG_WARNING,
+                   "Expected <longitude> XML element, received %s", node->name);
+            break;
+        }
 
-	xmlFree(content);
-	content = xmlNodeGetContent(node);
-	geo.lon = atof((const char *) content);
+        xmlFree(content);
+        content = xmlNodeGetContent(node);
+        geo.lon = atof((const char *) content);
 
-	value = icalvalue_new_geo(geo);
+        value = icalvalue_new_geo(geo);
 
-	break;
+        break;
     }
 
     case ICAL_PERIOD_VALUE: {
-	struct icalperiodtype p;
+        struct icalperiodtype p;
 
-	p.start = p.end = icaltime_null_time();
-	p.duration = icaldurationtype_from_int(0);
+        p.start = p.end = icaltime_null_time();
+        p.duration = icaldurationtype_from_int(0);
 
-	node = xmlFirstElementChild(xtype);
-	if (!node) {
-	    syslog(LOG_WARNING, "Missing <start> XML element");
-	    break;
-	}
-	else if (xmlStrcmp(node->name, BAD_CAST "start")) {
-	    syslog(LOG_WARNING,
-		   "Expected <start> XML element, received %s", node->name);
-	    break;
-	}
+        node = xmlFirstElementChild(xtype);
+        if (!node) {
+            syslog(LOG_WARNING, "Missing <start> XML element");
+            break;
+        }
+        else if (xmlStrcmp(node->name, BAD_CAST "start")) {
+            syslog(LOG_WARNING,
+                   "Expected <start> XML element, received %s", node->name);
+            break;
+        }
 
-	content = xmlNodeGetContent(node);
-	p.start = icaltime_from_string((const char *) content);
-	if (icaltime_is_null_time(p.start)) break;
+        content = xmlNodeGetContent(node);
+        p.start = icaltime_from_string((const char *) content);
+        if (icaltime_is_null_time(p.start)) break;
 
-	node = xmlNextElementSibling(node);
-	if (!node) {
-	    syslog(LOG_WARNING, "Missing <end> / <duration> XML element");
-	    break;
-	}
-	else if (!xmlStrcmp(node->name, BAD_CAST "end")) {
-	    xmlFree(content);
-	    content = xmlNodeGetContent(node);
-	    p.end = icaltime_from_string((const char *) content);
-	    if (icaltime_is_null_time(p.end)) break;
-	}
-	else if (!xmlStrcmp(node->name, BAD_CAST "duration")) {
-	    xmlFree(content);
-	    content = xmlNodeGetContent(node);
-	    p.duration = icaldurationtype_from_string((const char *) content);
-	    if (icaldurationtype_as_int(p.duration) == 0) break;
-	}
-	else {
-	    syslog(LOG_WARNING,
-		   "Expected <end> / <duration> XML element, received %s",
-		   node->name);
-	    break;
-	}
+        node = xmlNextElementSibling(node);
+        if (!node) {
+            syslog(LOG_WARNING, "Missing <end> / <duration> XML element");
+            break;
+        }
+        else if (!xmlStrcmp(node->name, BAD_CAST "end")) {
+            xmlFree(content);
+            content = xmlNodeGetContent(node);
+            p.end = icaltime_from_string((const char *) content);
+            if (icaltime_is_null_time(p.end)) break;
+        }
+        else if (!xmlStrcmp(node->name, BAD_CAST "duration")) {
+            xmlFree(content);
+            content = xmlNodeGetContent(node);
+            p.duration = icaldurationtype_from_string((const char *) content);
+            if (icaldurationtype_as_int(p.duration) == 0) break;
+        }
+        else {
+            syslog(LOG_WARNING,
+                   "Expected <end> / <duration> XML element, received %s",
+                   node->name);
+            break;
+        }
 
-	value = icalvalue_new_period(p);
+        value = icalvalue_new_period(p);
 
-	break;
+        break;
     }
 
     case ICAL_RECUR_VALUE: {
@@ -753,91 +760,91 @@ static icalvalue *xml_element_to_icalvalue(xmlNodePtr xtype,
 
         if (rt.freq != ICAL_NO_RECURRENCE) value = icalvalue_new_recur(rt);
 
-	break;
+        break;
     }
 
     case ICAL_REQUESTSTATUS_VALUE: {
-	struct icalreqstattype rst = { ICAL_UNKNOWN_STATUS, NULL, NULL };
-	short maj, min;
+        struct icalreqstattype rst = { ICAL_UNKNOWN_STATUS, NULL, NULL };
+        short maj, min;
 
-	node = xmlFirstElementChild(xtype);
-	if (!node) {
-	    syslog(LOG_WARNING, "Missing <code> XML element");
-	    break;
-	}
-	else if (xmlStrcmp(node->name, BAD_CAST "code")) {
-	    syslog(LOG_WARNING,
-		   "Expected <code> XML element, received %s", node->name);
-	    break;
-	}
+        node = xmlFirstElementChild(xtype);
+        if (!node) {
+            syslog(LOG_WARNING, "Missing <code> XML element");
+            break;
+        }
+        else if (xmlStrcmp(node->name, BAD_CAST "code")) {
+            syslog(LOG_WARNING,
+                   "Expected <code> XML element, received %s", node->name);
+            break;
+        }
 
-	content = xmlNodeGetContent(node);
-	if (sscanf((const char *) content, "%hd.%hd", &maj, &min) == 2) {
-	    rst.code = icalenum_num_to_reqstat(maj, min);
-	}
-	if (rst.code == ICAL_UNKNOWN_STATUS) {
-	    syslog(LOG_WARNING, "Unknown request-status code");
-	    break;
-	}
+        content = xmlNodeGetContent(node);
+        if (sscanf((const char *) content, "%hd.%hd", &maj, &min) == 2) {
+            rst.code = icalenum_num_to_reqstat(maj, min);
+        }
+        if (rst.code == ICAL_UNKNOWN_STATUS) {
+            syslog(LOG_WARNING, "Unknown request-status code");
+            break;
+        }
 
-	node = xmlNextElementSibling(node);
-	if (!node) {
-	    syslog(LOG_WARNING, "Missing <description> XML element");
-	    break;
-	}
-	else if (xmlStrcmp(node->name, BAD_CAST "description")) {
-	    syslog(LOG_WARNING,
-		   "Expected <description> XML element, received %s",
-		   node->name);
-	    break;
-	}
+        node = xmlNextElementSibling(node);
+        if (!node) {
+            syslog(LOG_WARNING, "Missing <description> XML element");
+            break;
+        }
+        else if (xmlStrcmp(node->name, BAD_CAST "description")) {
+            syslog(LOG_WARNING,
+                   "Expected <description> XML element, received %s",
+                   node->name);
+            break;
+        }
 
-	xmlFree(content);
-	content = xmlNodeGetContent(node);
-	rst.desc = (const char *) content;
+        xmlFree(content);
+        content = xmlNodeGetContent(node);
+        rst.desc = (const char *) content;
 
-	node = xmlNextElementSibling(node);
-	if (node) {
-	    if (xmlStrcmp(node->name, BAD_CAST "data")) {
-		syslog(LOG_WARNING,
-		       "Expected <data> XML element, received %s", node->name);
-		break;
-	    }
+        node = xmlNextElementSibling(node);
+        if (node) {
+            if (xmlStrcmp(node->name, BAD_CAST "data")) {
+                syslog(LOG_WARNING,
+                       "Expected <data> XML element, received %s", node->name);
+                break;
+            }
 
-	    xmlFree(content);
-	    content = xmlNodeGetContent(node);
-	    rst.debug = (const char *) content;
-	}
+            xmlFree(content);
+            content = xmlNodeGetContent(node);
+            rst.debug = (const char *) content;
+        }
 
-	value = icalvalue_new_requeststatus(rst);
-	break;
+        value = icalvalue_new_requeststatus(rst);
+        break;
     }
 
     case ICAL_UTCOFFSET_VALUE: {
-	int n, utcoffset, hours, minutes, seconds = 0;
-	char sign;
+        int n, utcoffset, hours, minutes, seconds = 0;
+        char sign;
 
-	content = xmlNodeGetContent(xtype);
-	n = sscanf((const char *) content, "%c%02d:%02d:%02d",
-		   &sign, &hours, &minutes, &seconds);
+        content = xmlNodeGetContent(xtype);
+        n = sscanf((const char *) content, "%c%02d:%02d:%02d",
+                   &sign, &hours, &minutes, &seconds);
 
-	if (n < 3) {
-	    syslog(LOG_WARNING, "Unexpected utc-offset format");
-	    break;
-	}
+        if (n < 3) {
+            syslog(LOG_WARNING, "Unexpected utc-offset format");
+            break;
+        }
 
-	utcoffset = hours*3600 + minutes*60 + seconds;
+        utcoffset = hours*3600 + minutes*60 + seconds;
 
-	if (sign == '-') utcoffset = -utcoffset;
+        if (sign == '-') utcoffset = -utcoffset;
 
-	value = icalvalue_new_utcoffset(utcoffset);
-	break;
+        value = icalvalue_new_utcoffset(utcoffset);
+        break;
     }
 
     default:
-	content = xmlNodeGetContent(xtype);
-	value = icalvalue_new_from_string(kind, (const char *) content);
-	break;
+        content = xmlNodeGetContent(xtype);
+        value = icalvalue_new_from_string(kind, (const char *) content);
+        break;
     }
 
     if (content) xmlFree(content);
@@ -862,15 +869,15 @@ static icalproperty *xml_element_to_icalproperty(xmlNodePtr xprop)
     propname = ucase(icalmemory_tmp_copy((const char *) xprop->name));
     kind = icalenum_string_to_property_kind(propname);
     if (kind == ICAL_NO_PROPERTY) {
-	syslog(LOG_WARNING, "Unknown xCal property type: %s", propname);
-	return NULL;
+        syslog(LOG_WARNING, "Unknown xCal property type: %s", propname);
+        return NULL;
     }
 
     /* Create new property */
     prop = icalproperty_new(kind);
     if (!prop) {
-	syslog(LOG_ERR, "Creation of new %s property failed", propname);
-	return NULL;
+        syslog(LOG_ERR, "Creation of new %s property failed", propname);
+        return NULL;
     }
     if (kind == ICAL_X_PROPERTY) icalproperty_set_x_name(prop, propname);
 
@@ -878,41 +885,41 @@ static icalproperty *xml_element_to_icalproperty(xmlNodePtr xprop)
     /* Add parameters */
     node = xmlFirstElementChild(xprop);
     if (node && !xmlStrcmp(node->name, BAD_CAST "parameters")) {
-	xmlNodePtr xparam;
+        xmlNodePtr xparam;
 
-	for (xparam = xmlFirstElementChild(node); xparam;
-	     xparam = xmlNextElementSibling(xparam)) {
-	    char *paramname =
-		ucase(icalmemory_tmp_copy((const char *) xparam->name));
-	    xmlChar *paramval = xmlNodeGetContent(xmlFirstElementChild(xparam));
+        for (xparam = xmlFirstElementChild(node); xparam;
+             xparam = xmlNextElementSibling(xparam)) {
+            char *paramname =
+                ucase(icalmemory_tmp_copy((const char *) xparam->name));
+            xmlChar *paramval = xmlNodeGetContent(xmlFirstElementChild(xparam));
 
-	    /* XXX  Need to handle multi-valued parameters */
-	    icalproperty_set_parameter_from_string(prop, paramname,
-						   (const char *) paramval);
+            /* XXX  Need to handle multi-valued parameters */
+            icalproperty_set_parameter_from_string(prop, paramname,
+                                                   (const char *) paramval);
 
-	    xmlFree(paramval);
-	}
+            xmlFree(paramval);
+        }
 
-	node = xmlNextElementSibling(node);
+        node = xmlNextElementSibling(node);
     }
 
     /* Get the value type */
     if (!node) {
-	syslog(LOG_WARNING, "Missing xCal value for %s property",
-	       propname);
-	return NULL;
+        syslog(LOG_WARNING, "Missing xCal value for %s property",
+               propname);
+        return NULL;
     }
     typestr = ucase(icalmemory_tmp_copy((const char *) node->name));
     valkind = !strcmp(typestr, "UNKNOWN") ? ICAL_X_VALUE :
-	icalenum_string_to_value_kind(typestr);
+        icalenum_string_to_value_kind(typestr);
     if (valkind == ICAL_NO_VALUE) {
-	syslog(LOG_WARNING, "Unknown xCal value type for %s property: %s",
-	       propname, typestr);
-	return NULL;
+        syslog(LOG_WARNING, "Unknown xCal value type for %s property: %s",
+               propname, typestr);
+        return NULL;
     }
     else if (valkind == ICAL_TEXT_VALUE) {
-	/* "text" also includes enumerated types - grab type from property */
-	valkind = icalproperty_kind_to_value_kind(kind);
+        /* "text" also includes enumerated types - grab type from property */
+        valkind = icalproperty_kind_to_value_kind(kind);
     }
 
 
@@ -921,43 +928,44 @@ static icalproperty *xml_element_to_icalproperty(xmlNodePtr xprop)
     case ICAL_CATEGORIES_PROPERTY:
     case ICAL_RESOURCES_PROPERTY:
     case ICAL_POLLPROPERTIES_PROPERTY:
-	if (valkind == ICAL_TEXT_VALUE) {
-	    /* Handle multi-valued properties */
-	    struct buf buf = BUF_INITIALIZER;
-	    xmlChar *content = NULL;
+        if (valkind == ICAL_TEXT_VALUE) {
+            /* Handle multi-valued properties */
+            struct buf buf = BUF_INITIALIZER;
+            xmlChar *content = NULL;
 
-	    content = xmlNodeGetContent(node);
-	    buf_setcstr(&buf, (const char *) content);
-	    free(content);
+            content = xmlNodeGetContent(node);
+            buf_setcstr(&buf, (const char *) content);
+            free(content);
 
-	    while ((node = xmlNextElementSibling(node))) {
-		buf_putc(&buf, ',');
-		content = xmlNodeGetContent(node);
-		buf_appendcstr(&buf, (const char *) content);
-		free(content);
-	    }
+            while ((node = xmlNextElementSibling(node))) {
+                buf_putc(&buf, ',');
+                content = xmlNodeGetContent(node);
+                buf_appendcstr(&buf, (const char *) content);
+                free(content);
+            }
 
-	    value = icalvalue_new_from_string(valkind, buf_cstring(&buf));
-	    buf_free(&buf);
-	    break;
-	}
+            value = icalvalue_new_from_string(valkind, buf_cstring(&buf));
+            buf_free(&buf);
+            break;
+        }
+        GCC_FALLTHROUGH
 
     default:
-	value = xml_element_to_icalvalue(node, valkind);
-	if (!value) {
-	    syslog(LOG_ERR, "Parsing %s property value failed", propname);
-	    goto error;
-	}
+        value = xml_element_to_icalvalue(node, valkind);
+        if (!value) {
+            syslog(LOG_ERR, "Parsing %s property value failed", propname);
+            goto error;
+        }
     }
 
     icalproperty_set_value(prop, value);
 
 
-    /* Sanity check */ 
+    /* Sanity check */
     if ((node = xmlNextElementSibling(node))) {
-	syslog(LOG_WARNING,
-	"Unexpected XML element in property: %s", node->name);
-	goto error;
+        syslog(LOG_WARNING,
+        "Unexpected XML element in property: %s", node->name);
+        goto error;
     }
 
     return prop;
@@ -981,59 +989,59 @@ static icalcomponent *xml_element_to_icalcomponent(xmlNodePtr xcomp)
 
     /* Get component type */
     kind =
-	icalenum_string_to_component_kind(
-	    ucase(icalmemory_tmp_copy((const char *) xcomp->name)));
+        icalenum_string_to_component_kind(
+            ucase(icalmemory_tmp_copy((const char *) xcomp->name)));
     if (kind == ICAL_NO_COMPONENT) {
-	syslog(LOG_WARNING, "Unknown xCal component type: %s", xcomp->name);
-	return NULL;
+        syslog(LOG_WARNING, "Unknown xCal component type: %s", xcomp->name);
+        return NULL;
     }
 
     /* Create new component */
     comp = icalcomponent_new(kind);
     if (!comp) {
-	syslog(LOG_ERR, "Creation of new %s component failed", xcomp->name);
-	return NULL;
+        syslog(LOG_ERR, "Creation of new %s component failed", xcomp->name);
+        return NULL;
     }
 
     /* Add properties */
     node = xmlFirstElementChild(xcomp);
     if (!node || xmlStrcmp(node->name, BAD_CAST "properties")) {
-	syslog(LOG_WARNING,
-	       "Expected <properties> XML element, received %s", node->name);
-	goto error;
+        syslog(LOG_WARNING,
+               "Expected <properties> XML element, received %s", node->name);
+        goto error;
     }
     for (xprop = xmlFirstElementChild(node); xprop;
-	 xprop = xmlNextElementSibling(xprop)) {
-	icalproperty *prop = xml_element_to_icalproperty(xprop);
+         xprop = xmlNextElementSibling(xprop)) {
+        icalproperty *prop = xml_element_to_icalproperty(xprop);
 
-	if (!prop) goto error;
+        if (!prop) goto error;
 
-	icalcomponent_add_property(comp, prop);
+        icalcomponent_add_property(comp, prop);
     }
 
     /* Add sub-components */
     if (!(node = xmlNextElementSibling(node))) return comp;
 
     if (xmlStrcmp(node->name, BAD_CAST "components")) {
-	syslog(LOG_WARNING,
-	       "Expected <components> XML element, received %s", node->name);
-	goto error;
+        syslog(LOG_WARNING,
+               "Expected <components> XML element, received %s", node->name);
+        goto error;
     }
 
     for (xsub = xmlFirstElementChild(node); xsub;
-	 xsub = xmlNextElementSibling(xsub)) {
-	icalcomponent *sub = xml_element_to_icalcomponent(xsub);
+         xsub = xmlNextElementSibling(xsub)) {
+        icalcomponent *sub = xml_element_to_icalcomponent(xsub);
 
-	if (!sub) goto error;
+        if (!sub) goto error;
 
-	icalcomponent_add_component(comp, sub);
+        icalcomponent_add_component(comp, sub);
     }
 
-    /* Sanity check */ 
+    /* Sanity check */
     if ((node = xmlNextElementSibling(node))) {
-	syslog(LOG_WARNING,
-	"Unexpected XML element in component: %s", node->name);
-	goto error;
+        syslog(LOG_WARNING,
+        "Unexpected XML element in component: %s", node->name);
+        goto error;
     }
 
     return comp;
@@ -1047,34 +1055,34 @@ static icalcomponent *xml_element_to_icalcomponent(xmlNodePtr xcomp)
 /*
  * Construct an iCalendar component from an xCal string.
  */
-icalcomponent *xcal_string_as_icalcomponent(const char *str)
+icalcomponent *xcal_string_as_icalcomponent(const struct buf *buf)
 {
     xmlParserCtxtPtr ctxt;
     xmlDocPtr doc = NULL;
     xmlNodePtr root;
     icalcomponent *ical = NULL;
 
-    if (!str) return NULL;
+    if (!buf_len(buf)) return NULL;
 
     /* Parse the XML request */
     ctxt = xmlNewParserCtxt();
     if (ctxt) {
-	doc = xmlCtxtReadMemory(ctxt, str, strlen(str), NULL, NULL,
-				XML_PARSE_NOWARNING);
-	xmlFreeParserCtxt(ctxt);
+        doc = xmlCtxtReadMemory(ctxt, buf_base(buf), buf_len(buf), NULL, NULL,
+                                XML_PARSE_NOWARNING);
+        xmlFreeParserCtxt(ctxt);
     }
     if (!doc) {
-	syslog(LOG_WARNING, "XML parse error");
-	return NULL;
+        syslog(LOG_WARNING, "XML parse error");
+        return NULL;
     }
 
     /* Get root element */
     if (!(root = xmlDocGetRootElement(doc)) ||
-	xmlStrcmp(root->name, BAD_CAST "icalendar") ||
-	xmlStrcmp(root->ns->href, BAD_CAST XML_NS_ICALENDAR)) {
-	syslog(LOG_WARNING,
-	       "XML root element is not %s:icalendar", XML_NS_ICALENDAR);
-	goto done;
+        xmlStrcmp(root->name, BAD_CAST "icalendar") ||
+        xmlStrcmp(root->ns->href, BAD_CAST XML_NS_ICALENDAR)) {
+        syslog(LOG_WARNING,
+               "XML root element is not %s:icalendar", XML_NS_ICALENDAR);
+        goto done;
     }
 
     ical = xml_element_to_icalcomponent(xmlFirstElementChild(root));
@@ -1096,7 +1104,7 @@ const char *begin_xcal(struct buf *buf)
     buf_printf_markup(buf, 2, "<properties>");
     buf_printf_markup(buf, 3, "<prodid>");
     buf_printf_markup(buf, 4, "<text>-//CyrusIMAP.org/Cyrus %s//EN</text>",
-		      cyrus_version());
+                      cyrus_version());
     buf_printf_markup(buf, 3, "</prodid>");
     buf_printf_markup(buf, 3, "<version>");
     buf_printf_markup(buf, 4, "<text>2.0</text>");
@@ -1116,46 +1124,3 @@ void end_xcal(struct buf *buf)
     buf_printf_markup(buf, 1, "</vcalendar>");
     buf_printf_markup(buf, 0, "</icalendar>");
 }
-
-
-/* libxml2 replacement functions for those missing in older versions */
-#if (LIBXML_VERSION < 20800)
-xmlChar *xmlBufferDetach(xmlBufferPtr buf)
-{
-    xmlChar *ret;
-
-    if (!buf) return NULL;
-
-    ret = buf->content;
-    buf->content = NULL;
-    buf->use = buf->size = 0;
-
-    return ret;
-}
-
-
-#if (LIBXML_VERSION < 20703)
-xmlNodePtr xmlFirstElementChild(xmlNodePtr node)
-{
-    if (!node) return NULL;
-
-    for (node = node->children; node; node = node->next) {
-	if (node->type == XML_ELEMENT_NODE) return (node);
-    }
-
-    return NULL;
-}
-
-
-xmlNodePtr xmlNextElementSibling(xmlNodePtr node)
-{
-    if (!node) return NULL;
-
-    for (node = node->next; node; node = node->next) {
-	if (node->type == XML_ELEMENT_NODE) return (node);
-    }
-
-    return NULL;
-}
-#endif /* < 2.7.3 */
-#endif /* < 2.8.0 */
