@@ -174,7 +174,7 @@ static char* look_for_me(char *myaddr, int numaddresses,
         struct address_itr ai;
         const struct address *a;
 
-        address_itr_init(&ai, body[l]);
+        address_itr_init(&ai, body[l], 0);
 
         /* loop through each address in the header */
         while (!found && (a = address_itr_next(&ai)) != NULL) {
@@ -574,6 +574,7 @@ static int eval_bc_test(sieve_interp_t *interp, void* m, void *sc,
         for (x=0; x<numheaders && !res; x++)
         {
             const char *this_header;
+            int reverse_path = 0;
 
             currh = unwrap_string(bc, currh, &this_header, NULL);
 
@@ -593,6 +594,8 @@ static int eval_bc_test(sieve_interp_t *interp, void* m, void *sc,
                 /* Envelope */
                 if(interp->getenvelope(m, this_header, &val) != SIEVE_OK)
                     continue;
+
+                if (!strcmp(this_header, "from")) reverse_path = 1;
             }
 
             /* count results */
@@ -627,7 +630,7 @@ static int eval_bc_test(sieve_interp_t *interp, void* m, void *sc,
                 printf("about to parse %s\n", val[y]);
 #endif
 
-                address_itr_init(&ai, val[y]);
+                address_itr_init(&ai, val[y], reverse_path);
 
                 while (!res && (a = address_itr_next(&ai)) != NULL) {
 #if VERBOSE
@@ -914,6 +917,8 @@ envelope_err:
     case BC_STRING:/*21*/
         is_string = 1;
 
+        GCC_FALLTHROUGH
+
     case BC_HASFLAG:/*15*/
     {
         int haystacksi=i+4;/*the i value for the beginning of the variables*/
@@ -1164,15 +1169,14 @@ envelope_err:
 
         /*find the part(s) of the body that we want*/
         content_types = bc_makeArray(bc, &typesi);
-        if(interp->getbody(m, content_types, &val) != SIEVE_OK) {
-            res = SIEVE_RUN_ERROR;
-            break;
-        }
+        res = interp->getbody(m, content_types, &val);
         free(content_types);
 
+        if (res != SIEVE_OK) break;
+
         /* bodypart(s) exist, now to test them */
 
-        for (y = 0; val && val[y] && !res; y++) {
+        for (y = 0; val && val[y]; y++) {
 
             if (match == B_COUNT) {
                 count++;
@@ -1195,9 +1199,9 @@ envelope_err:
                         reg = bc_compile_regex(data_val, ctag,
                                                errbuf, sizeof(errbuf));
                         if (!reg) {
-                            /* Oops */
-                            res=-1;
-                            goto alldone;
+                            /* should only get here due to a memory issue */
+                            res = SIEVE_NOMEM;
+                            break;
                         }
 
                         res |= comp(content, strlen(content), (const char *)reg, comprock);
@@ -1242,6 +1246,8 @@ envelope_err:
     }
     case BC_DATE:/*11*/
         has_index=1;
+        GCC_FALLTHROUGH
+
     case BC_CURRENTDATE:/*12*/
         if (0x07 == version) {
             /* There was a version of the bytecode that had the index extension
